#include <chrono>
#include <iostream>
#include <locale>

#include <cstdlib>

extern "C" {
	#include <sched.h>
	#include <unistd.h>
}

using std::chrono::steady_clock,
      std::chrono::microseconds,
      std::chrono::seconds;

int
main(int argc, char ** argv)
{
	seconds const runtime{30};
	size_t warmup{100};

	if (argc != 2) {
		std::cerr << "Syntax:" << std::endl
		          << argv[0] << " (Y|P) ... run in yielding/polling mode" << std::endl;
		std::exit(EXIT_FAILURE);
	}

	bool yield;
	switch (argv[1][0]) {
		case 'p': case 'P':
			yield = false;
			break;
		case 'y': case 'Y':
			yield = true;
			break;
		default:
			std::cerr << "Unknown option: " << argv[1] << std::endl;
			std::exit(EXIT_FAILURE);
	}

	// warm up CPU
	for (size_t cc{0}; cc < warmup; ++cc)
	{
		[[maybe_unused]] volatile auto dummy {steady_clock::now()};
	}

	std::chrono::time_point const target { steady_clock::now() + runtime };
	ssize_t counter{0};
	while (steady_clock::now() < target)
	{
		++counter;

		// depending on our arguments...
		if (yield) {
			// ... yield
			sched_yield();
		} else {
			// ... do some other system call to generate a similar overhead
			[[maybe_unused]] volatile pid_t dummy{getppid()};
		}
	}

	std::cout.imbue(std::locale("")); // use locale from environment
	std::cout << "PID " << getpid() << " did " << (yield ? "yield" : "poll")
	          << " and reached counter " << counter << std::endl;
}
